<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:tdm="http://www.quest.com/toad-data-modeler" xmlns:msxsl="urn:schemas-microsoft-com:xslt">
	<tdm:description>Generation of customized XSD files.</tdm:description>
	<tdm:caption>XML Schema (XSD) - Elements</tdm:caption>
	<tdm:default-filename>XMLSchema-Elements</tdm:default-filename>
	<tdm:default-suffix>xsd</tdm:default-suffix>
	<tdm:db-platforms>
		<tdm:db-platform>OR9</tdm:db-platform>
		<tdm:db-platform>OR10</tdm:db-platform>
		<tdm:db-platform>OR11</tdm:db-platform>
	</tdm:db-platforms>
	<xsl:output version="4.0" method="xml" indent="yes" omit-xml-declaration="no" encoding="UTF-8" />
	<xsl:param name="targetNamespace" select="namespace" />
	<xsl:param name="RootElement" select="Tables" />
	<xsl:template match="/">
		<xsl:text disable-output-escaping="yes">&lt;xs:schema attributeFormDefault="unqualified" targetNamespace="</xsl:text>
		<xsl:value-of select="$targetNamespace" />
		<xsl:text disable-output-escaping="yes">" xmlns:xs="http://www.w3.org/2001/XMLSchema"&gt;</xsl:text>

		<!--*************************************************************-->
		<!--Create root element-->
		<!--*************************************************************-->
		<xsl:text disable-output-escaping="yes">&lt;xs:element name="</xsl:text>
		<xsl:value-of select="$RootElement" />
		<xsl:text disable-output-escaping="yes">"&gt;</xsl:text>
		<xsl:text disable-output-escaping="yes">&lt;xs:complexType</xsl:text>
		<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
		<xsl:text disable-output-escaping="yes">&lt;xs:sequence</xsl:text>
		<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
		
		<!--*************************************************************-->
		<!--Iterate entities and create new element for each entity-->
		<!--*************************************************************-->
		<xsl:for-each select="/Model/Entities/Entity">
			<xsl:sort />
			<xsl:text disable-output-escaping="yes">&lt;xs:element name="</xsl:text>
			<xsl:value-of select="Name" />
			<xsl:text disable-output-escaping="yes">"</xsl:text>
			<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
			<xsl:text disable-output-escaping="yes">&lt;xs:complexType</xsl:text>
			<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
			<xsl:text disable-output-escaping="yes">&lt;xs:sequence</xsl:text>
			<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
			
			<!--*************************************************************-->
			<!--Iterate attributes and create new element for each attribute.-->
			<!--If standard data type is used, template DataTypeConversion-->
			<!--is called. If domain is used, template DomainDetails is called-->
			<!--first. If special data type is used, anyType will be returned.-->
			<!--*************************************************************-->
			<xsl:for-each select="Attributes/Attribute">
				<xsl:text disable-output-escaping="yes">&lt;xs:element name="</xsl:text>
				<xsl:value-of select="Name" />
				<xsl:text disable-output-escaping="yes">" type="</xsl:text>
				<xsl:choose>
					<xsl:when test="string-length(DataType/Name)>0">
						<xsl:call-template name="DataTypeConversion">
							<xsl:with-param name="datatype" select="DataType/Name" />
						</xsl:call-template>
					</xsl:when>
					<xsl:when test="string-length(Domain/Name)>0">
						<xsl:call-template name="DomainDetails">
							<xsl:with-param name="domainid" select="Domain/Id" />
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text disable-output-escaping="yes">xs:anyURI</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:text disable-output-escaping="yes">"</xsl:text>
				<xsl:choose>
					<xsl:when test="NotNull = 1">
						<xsl:text disable-output-escaping="yes"> minOccurs="1" maxOccurs="1" </xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text disable-output-escaping="yes"> minOccurs="0" maxOccurs="1" nillable="true" </xsl:text>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:text disable-output-escaping="yes">/&gt;</xsl:text>
			</xsl:for-each>
			<xsl:text disable-output-escaping="yes">&lt;/xs:sequence</xsl:text>
			<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
			<xsl:text disable-output-escaping="yes">&lt;/xs:complexType</xsl:text>
			<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
			<xsl:text disable-output-escaping="yes">&lt;/xs:element</xsl:text>
			<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
		</xsl:for-each>
		<xsl:text disable-output-escaping="yes">&lt;/xs:sequence</xsl:text>
		<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
		<xsl:text disable-output-escaping="yes">&lt;/xs:complexType</xsl:text>
		<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
		<xsl:text disable-output-escaping="yes">&lt;/xs:element</xsl:text>
		<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
		<xsl:text disable-output-escaping="yes">&lt;/xs:schema</xsl:text>
		<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
	</xsl:template>

	<!--*************************************************************-->
	<!--Template DataTypeConversion defines conversion rules.-->
	<!--*************************************************************-->
	<xsl:template name="DataTypeConversion">
		<xsl:param name="datatype" />
		<xsl:choose>
			<xsl:when test="contains($datatype,'Bfile') or contains($datatype,'Blob') or contains($datatype,'Raw') or contains($datatype,'raw')">
				<xsl:text disable-output-escaping="yes">xs:base64binary</xsl:text>
			</xsl:when>
			<xsl:when test="contains($datatype,'Date')">
				<xsl:text disable-output-escaping="yes">xs:date</xsl:text>
			</xsl:when>
			<xsl:when test="contains($datatype,'Number') or contains($datatype,'Real')">
				<xsl:text disable-output-escaping="yes">xs:decimal</xsl:text>
			</xsl:when>
			<xsl:when test="contains($datatype,'Binary_Double')">
				<xsl:text disable-output-escaping="yes">xs:double</xsl:text>
			</xsl:when>
			<xsl:when test="contains($datatype,'Timestamp')">
				<xsl:text disable-output-escaping="yes">xs:duration</xsl:text>
			</xsl:when>			
			<xsl:when test="contains($datatype,'Float')">
				<xsl:text disable-output-escaping="yes">xs:float</xsl:text>
			</xsl:when>
			<xsl:when test="contains($datatype,'Integer')">
				<xsl:text disable-output-escaping="yes">xs:int</xsl:text>
			</xsl:when>
			<xsl:when test="contains($datatype,'Clob') or contains($datatype,'Char')  or contains($datatype,'Long') or contains($datatype,'Varchar') or contains($datatype,'Rowid') or contains($datatype,'Urowid')">
				<xsl:text disable-output-escaping="yes">xs:string</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text disable-output-escaping="yes">xs:anyURI</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!--*************************************************************-->
	<!--Template DomainDetails works with Domain tags.-->
	<!--*************************************************************-->
	<xsl:template name="DomainDetails">
		<xsl:param name="domainid" />
		<xsl:for-each select="/Model/Domains/Domain">
			<xsl:if test="Id=$domainid">
				<xsl:call-template name="DataTypeConversion">
					<xsl:with-param name="datatype" select="DataType/Name" />
				</xsl:call-template>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
</xsl:stylesheet>